import app from 'flarum/admin/app';
import Page from 'flarum/common/components/Page';
import Button from 'flarum/common/components/Button';
import Select from 'flarum/common/components/Select';
import Checkbox from 'flarum/common/components/Checkbox';
import type Scope from 'src/common/models/Scope';
import LoadingIndicator from 'flarum/common/components/LoadingIndicator';
import Stream from 'flarum/common/utils/Stream';
import { randomString } from '../util';

interface EditingScope {
  model?: Scope,
  markDeletion?: boolean,
  modifiedFields: string[],
  scope?: Stream,
  resource_path?: Stream,
  method?: Stream,
  is_default?: Stream,
  scope_name?: Stream,
  scope_icon?: Stream,
  scope_desc?: Stream
}

export default class ScopesPage extends Page {
  translationPrefix = 'rhodes-island-oauth-center.admin.scopes.';
  scopes: Scope[] = [];
  fields = [
    'scope',
    'resource_path',
    'method',
    'is_default',
    'scope_name',
    'scope_icon',
    'scope_desc'
  ];

  bufferedValues: EditingScope[] = [];

  components: { [x: string]: (stream: Stream, disabled: boolean) => any } = {
    "method": (stream, disabled) => <Select options={{
      "GET": "GET",
      "POST": "POST",
      "PUT": "PUT",
      "DELETE": "DELETE",
      "PATCH": "PATCH"
    }} disabled={disabled} value={stream()} onchange={(val: boolean) => stream(val)}></Select>,
    "is_default": (stream, disabled) => <Checkbox disabled={disabled} state={stream() === 1} onchange={(val: boolean) => stream(val ? 1 : 0)}></Checkbox>
  }

  loading = false;
  saving = false;

  oninit(vnode: any) {
    super.oninit(vnode);

    this.refresh();
  }

  streamMapper(stream: Stream, scope: EditingScope, field: string) {
    stream.map((val: any) => {
      if (stream.notInitial) {
        scope.modifiedFields.push(field);
      } else {
        stream.notInitial = true;
      }
      return val;
    });
  }

  refresh() {
    this.loading = true;
    this.bufferedValues = [];
    app.store.find('oauth-scopes').then(r => {
      this.scopes = r as unknown as Scope[];
      for (const scope of this.scopes) {
        const eScope: EditingScope = {model: scope, modifiedFields: []};
        for (const field of this.fields) {
          const value = (scope as any)[field]();
          const stream = Stream(value ? value : (field == "method" ? "GET" : null));
          (eScope as any)[field] = stream;
          this.streamMapper(stream, eScope, field);
        }
        this.bufferedValues.push(eScope);
      }
      this.loading = false;
      m.redraw();
    });
  }

  view() {
    return (
      <div class="OAuthCenter-scopesPage">
        {(this.loading ? <LoadingIndicator></LoadingIndicator> : <div className="Form-group">
          <table>
            <thead>
              <tr>
                {this.fields.map(key => <th>{app.translator.trans(this.translationPrefix + key)}</th>)}
              </tr>
            </thead>
            <tbody>
              {this.bufferedValues.map((scope) => <tr style={scope.markDeletion ? "opacity: .5;" : ""}>
                {this.fields.map((key) => <td>
                  {(this.components[key] && this.components[key]((scope as any)[key], this.saving)) ?? <input className="FormControl" type="text" bidi={(scope as any)[key]} disabled={this.saving}></input>}
                </td>)}
                <td>
                  <Button className="Button Button--icon" disabled={this.saving} icon={"fas fa-" + (scope.markDeletion ? "undo" : "times")} onclick={() => {
                    scope.markDeletion = !scope.markDeletion;
                  }}></Button>
                </td>
              </tr>)}
            </tbody>
          </table>
          <Button className="Button Button--block" disabled={this.saving} onclick={() => {
            const eScope: EditingScope = {modifiedFields: []};
            for (const field of this.fields) {
              const stream = Stream();
              (eScope as any)[field] = stream;
            }
            eScope.scope("scope." + randomString(4));
            eScope.method("GET");
            this.bufferedValues.push(eScope);
          }}>{app.translator.trans(this.translationPrefix + 'add_button')}</Button>
          <br/>
          <Button className="Button Button--primary" onclick={this.save.bind(this)} loading={this.saving}>{app.translator.trans(this.translationPrefix + "save")}</Button>
        </div>)}
      </div>
    );
  }

  save() {
    const promises = [];
    for (const eScope of this.bufferedValues) {
      const scope = eScope.model;
      if (scope) {
        if (eScope.markDeletion) {
          promises.push(scope.delete());
        } else {
          if (eScope.modifiedFields.length > 0) {
            const attrs: any = {};
            for (const field of eScope.modifiedFields) {
              attrs[field] = (eScope as any)[field]();
            }
            promises.push(scope.save(attrs));
          }
        }
      } else {
        const scope = app.store.createRecord("oauth-scopes");
        const attrs: any = {};
        for (const field of this.fields) {
          const value = (eScope as any)[field]();
          if (value) {
            attrs[field] = value;
          }
        }
        promises.push(scope.save(attrs));
      }
    }
    if (promises.length > 0) {
      this.saving = true;
      Promise.allSettled(promises).finally(() => {
        this.saving = false;
        this.refresh();
      });
    }
  }
}
